/*
 * InitDMA.c
 *
 *  Created on: 2011-9-2
 *      Author: ChengSi
 */
#include <time.h>
#include <csl_edma3.h>
#include <stdio.h>

/*Channel Controller Registers*/
#define TPCC_BASEADDR(n)			(0x02700000 + n * 0x20000)

#define DCHMAP_BASEADDR(n)			(TPCC_BASEADDR(n) + 0x0100)
#define DCHMAP						((volatile int*)DCHMAP_BASEADDR(n))
#define DMAQNUM_BASEADDR(n)			(TPCC_BASEADDR(n) + 0x0240)
#define DMAQNUM						((volatile int*)DMAQNUM_BASEADDR(n))
#define QUEPRI(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x0284)
#define EMR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x0300)
#define EMRH(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x0304)
#define EMCR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x0308)
#define EMCRH(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x030C)
#define CCERR(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x0318)
#define CCERRCLR(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x031C)
#define DRAE0(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x0340)
#define DRAEH0(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x0344)
#define QWMTHRA(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x0620)

/*Global Registers*/
#define ER(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1000)
#define ERH(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1004)
#define ECR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1008)
#define ECRH(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x100C)
#define ESR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1010)
#define CER(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1018)
#define CERH(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x101C)
#define EER(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1020)
#define EERH(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1024)
#define EECR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1028)
#define EECRH(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x102C)
#define EESR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1030)
#define EESRH(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x1034)
#define SER(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1038)
#define SERH(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x103C)
#define SECR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1040)
#define SECRH(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x1044)
#define IER(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1050)
#define IERH(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1054)
#define IECR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1058)
#define IECRH(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x105C)
#define IESR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1060)
#define IESRH(n)					*(volatile int*)(TPCC_BASEADDR(n) + 0x1064)
#define IPR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1068)
#define IPRH(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x106C)
#define ICR(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1070)
#define ICRH(n)						*(volatile int*)(TPCC_BASEADDR(n) + 0x1074)

/*PaRAM,nʾnͨmʾͨmPaRAM*/
#define PaRAM_BASEADDR(n,m)			(TPCC_BASEADDR(n) + 0x4000 + 0x20 * m)
#define OPT(n,m)					*(volatile int*)PaRAM_BASEADDR(n,m)
#define SRC(n,m)					*(volatile int*)(PaRAM_BASEADDR(n,m) + 0x4)
#define ABCNT(n,m)					*(volatile int*)(PaRAM_BASEADDR(n,m) + 0x8)
#define DST(n,m)					*(volatile int*)(PaRAM_BASEADDR(n,m) + 0xc)
#define BIDX(n,m)					*(volatile int*)(PaRAM_BASEADDR(n,m) + 0x10)
#define RLD(n,m)					*(volatile int*)(PaRAM_BASEADDR(n,m) + 0x14)
#define CIDX(n,m)					*(volatile int*)(PaRAM_BASEADDR(n,m) + 0x18)
#define CCNT(n,m)					*(volatile int*)(PaRAM_BASEADDR(n,m) + 0x1c)


void InitDMA(int n)
{
	IECR(n)			= 0xFFFFFFFF;
	IECRH(n)		= 0xFFFFFFFF;
	EECR(n)			= 0xFFFFFFFF;
	EECRH(n)		= 0xFFFFFFFF;
	EMCR(n)			= 0xFFFFFFFF;
	EMCRH(n)		= 0xFFFFFFFF;
	CCERRCLR(n)		= 0xFFFFFFFF;
	ICR(n)   		= 0xFFFFFFFF;
	ICRH(n)   		= 0xFFFFFFFF;
	SECR(n)			= 0xFFFFFFFF;
	SECRH(n)		= 0xFFFFFFFF;
	QUEPRI(n)		= 0x3210;
	QWMTHRA(n)		= 0x10101010;

	DMAQNUM[0]		= 0;
	DMAQNUM[1]		= 0;
	DMAQNUM[2]		= 0;
	DMAQNUM[3]		= 0;
	DMAQNUM[4]		= 0;
	DMAQNUM[5]		= 0;
	DMAQNUM[6]		= 0;
	DMAQNUM[7]		= 0;
	DCHMAP[2] 		= 0;			//2ͨӦPaRam0

	EESR(n)		= 1<<2;
	IESR(n)		= 1;
	DRAE0(n) 	= 1;
}

//DMA豸豸ʱַҪȫֵַʹñصַ
void DMA_Transfer(void* dst, void* src, int n)
{
	int t1 = 0, t2 = 0;
	ICR(n)   = 1;
	EMCR(n)  = 1<<2;
	EMCR(n)  = 1<<2;
	SECR(n)  = 1<<2;
	SECRH(n) = 1<<2;
	CCERRCLR(n)	= 0xFFFFFFFF;

	OPT(n,0) = CSL_EDMA3_OPT_MAKE(
				CSL_EDMA3_ITCCH_DIS,\
				CSL_EDMA3_TCCH_DIS,\
				CSL_EDMA3_ITCINT_DIS,\
				CSL_EDMA3_TCINT_EN,\
				0,\
				CSL_EDMA3_TCC_NORMAL,\
				CSL_EDMA3_FIFOWIDTH_NONE,\
				CSL_EDMA3_STATIC_DIS,\
				CSL_EDMA3_SYNC_AB,\
				CSL_EDMA3_ADDRMODE_INCR,\
				CSL_EDMA3_ADDRMODE_INCR
				);
	DST(n,0) 	= (int)dst;
	SRC(n,0) 	= (int)src;
	ABCNT(n,0)	= CSL_EDMA3_CNT_MAKE(1024 ,100);
	BIDX(n,0)	= CSL_EDMA3_BIDX_MAKE(1024,1024);
	RLD(n,0)	= CSL_EDMA3_LINKBCNTRLD_MAKE(CSL_EDMA3_LINK_NULL,0);
	CIDX(n,0)	= CSL_EDMA3_CIDX_MAKE(0,0);
	CCNT(n,0)	= 1;

	t1 = clock();
	ESR(n) 		= 1<<2;	//ֶDMA
	while((IPR(n) & 1) != 1);
	t2 = clock();
	printf("t2 - t1 = %d\n",(t2 - t1));
}

